@@ -24,6 +24,7 @@
 #include "../css.h"
 #include "../layout.h"
 #include "../render.h"
+#include "../scheme.h"
 #include "../util.h"
 #include "../window.h"
 #include "../xul.h"
@@ -41,7 +42,7 @@ SCM ruin_scm_attribute_invert_foreground_color(SCM color) {
   SCM_ASSERT(SCM_STRINGP(color), color, SCM_ARG1, 
 	     "ruin:invert-foreground-color");
 
-  input_color = SCM_STRING_CHARS(color);
+  input_color = scm_to_locale_string(color);
   matched_color = ruin_css_match_foreground_color(input_color);
   switch(matched_color) {
   case RUIN_LAYOUT_FG_COLOR_BLACK: 
@@ -114,7 +115,7 @@ SCM ruin_scm_attribute_invert_background_color(SCM color) {
   SCM_ASSERT(SCM_STRINGP(color), color, SCM_ARG1, 
 	     "ruin:invert-background-color");
 
-  input_color = SCM_STRING_CHARS(color);
+  input_color = scm_to_locale_string(color);
   matched_color = ruin_css_match_background_color(input_color, NULL);
   switch(matched_color) {
   case RUIN_LAYOUT_BG_COLOR_BLACK: 
