@@ -18,6 +18,7 @@
 
 #include <libguile.h>
 
+#include "../css.h"
 #include "../layout.h"
 #include "../util.h"
 #include "../window.h"
@@ -40,6 +41,9 @@ SCM ruin_scm_api_window_focus_next(SCM selt) {
 
   len = ruin_util_list_length(w->tab_order);
 
+  if (len == 0)
+    return SCM_BOOL_F;
+
   for (; i < len; i++) {
     ruin_element_t *t = (ruin_element_t *) 
       ruin_util_string_to_ptr(ruin_util_list_peek_after(w->tab_order, i));
@@ -68,6 +72,9 @@ SCM ruin_scm_api_window_focus_prev(SCM selt) {
 
   len = ruin_util_list_length(w->tab_order);
 
+  if (len == 0)
+    return SCM_BOOL_F;
+
   for (; i < len; i++) {
     ruin_element_t *t = (ruin_element_t *) 
       ruin_util_string_to_ptr(ruin_util_list_peek_after(w->tab_order, i));
@@ -128,6 +135,9 @@ SCM ruin_scm_api_window_focus(SCM selt) {
 		 w->focused->element, 
 		 scm_str2symbol("sdom:event-dom-focus-in"), 
 		 SCM_EOL, SCM_EOL);
+
+      ruin_css_clear_style_cache(w->focused);
+      ruin_scm_api_window_render(w->focused->element);
     }
   }
   return SCM_BOOL_F;
