@@ -1,5 +1,5 @@
 /* css.c: Cascade-management routines for libRUIN
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,12 +16,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 
+#include <libguile.h>
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "css.h"
 #include "layout.h"
+#include "libruin.h"
+#include "scheme.h"
 #include "util.h"
 
 /* The orders here have to match the orders of the enumerations in the 
@@ -52,19 +55,21 @@ static SCM scsc_p = SCM_EOL;
 
 int ruin_css_get_rgb(char *c) {
   SCM sc = SCM_EOL;
-  SCM val = SCM_MAKINUM(0);
+  SCM val = scm_from_int32(0);
 
   if (sch_p == SCM_EOL) {
     sch_p = scm_c_eval_string("scss:color->hex");
   }
 
   if (c[0] == '#')
-    val = scm_string_to_number(scm_makfrom0str(c + 1), SCM_MAKINUM(16));
+    val = scm_string_to_number(scm_makfrom0str(c + 1), scm_from_int32(16));
   else {
     sc = scm_call_1(sch_p, scm_makfrom0str(c));
     if (scm_eq_p(sc, SCM_EOL) != SCM_BOOL_T) 
-      val = scm_string_to_number
-	(scm_substring(sc, SCM_MAKINUM(1), SCM_MAKINUM(7)), SCM_MAKINUM(16));
+      val = scm_string_to_number(scm_substring(sc,
+					       scm_from_int32(1), 
+					       scm_from_int32(7)), 
+				 scm_from_int32(16));
   }
   return scm_num2int(val, 0, "libruin");
 }
@@ -165,11 +170,13 @@ char *ruin_css_lookup(ruin_element_t *tree, char *prop) {
     if (tree->dialect == RUIN_XML_DIALECT_XHTML) {
       backup_sheet = SCM_CADDR(tree->cascade);
       scm_call_2(ssas_p, tree->cascade, 
-		 scm_cons(tree->inherent_attribute_style, backup_sheet));
+		 scm_cons(SCM_CAR(tree->inherent_attribute_style), 
+			  backup_sheet));
     } else {
       backup_sheet = SCM_CADR(tree->cascade);
       scm_call_2(ssuas_p, tree->cascade,
-		 scm_cons(tree->inherent_attribute_style, backup_sheet));
+		 scm_cons(SCM_CAR(tree->inherent_attribute_style),
+			  backup_sheet));
     }
   }
 
@@ -179,7 +186,7 @@ char *ruin_css_lookup(ruin_element_t *tree, char *prop) {
       (ssvan_p, tree->cascade, tree->element, tree->doc, scm_prop);
     full_value_select_time += ruin_util_current_time_millis() - s;
     full_value_select_num++;
-  } else if (strcmp(SCM_STRING_CHARS(tree->element), 
+  } else if (strcmp(scm_to_locale_string(tree->element), 
 		    "ruin-inline-element") == 0) {
     if (strcmp(prop, "display") == 0)
       cascade_value = scm_makfrom0str("inline");
@@ -201,29 +208,55 @@ char *ruin_css_lookup(ruin_element_t *tree, char *prop) {
     scm_call_2(tree->dialect == RUIN_XML_DIALECT_XHTML ? ssas_p : ssuas_p, 
 	       tree->cascade, backup_sheet);
   }
-  if (scm_eq_p(tree->additional_attribute_style, SCM_EOL) != SCM_BOOL_T) {
-    long s = ruin_util_current_time_millis();
-    SCM eas_res = scm_call_3
-      (ssvan_p, scm_list_3(SCM_EOL, SCM_EOL, 
-			   scm_list_1(tree->additional_attribute_style)), 
-       scm_makfrom0str("ruin-additional-style-lookup"), scm_prop);
-    full_value_select_time += ruin_util_current_time_millis() - s;
-    full_value_select_num++;
-    if (scm_eq_p(eas_res, SCM_EOL) != SCM_BOOL_T)
-      cascade_value = eas_res;
+
+  { ruin_element_t *as_location = NULL;
+    if (scm_eq_p(tree->additional_attribute_style, SCM_EOL) != SCM_BOOL_T) {
+      as_location = tree;
+    } else if (tree->content != NULL &&
+	       scm_eq_p(tree->parent->additional_attribute_style, SCM_EOL) !=
+	       SCM_BOOL_T) {
+      
+      /* If the element is an inline element with content, we need to check
+	 whether the parent element has additional attribute style. */
+      
+      as_location = tree->parent;
+    }
+    if (as_location != NULL) {
+      long s = ruin_util_current_time_millis();
+      SCM eas_res = scm_call_3
+	(ssvan_p, 
+	 scm_list_3(SCM_EOL, SCM_EOL, as_location->additional_attribute_style),
+	 scm_makfrom0str("ruin-additional-style-lookup"), scm_prop);
+      full_value_select_time += ruin_util_current_time_millis() - s;
+      full_value_select_num++;
+      if (scm_eq_p(eas_res, SCM_EOL) != SCM_BOOL_T)
+	cascade_value = eas_res;
+    }
   }
+
   if (scm_eq_p(cascade_value, SCM_EOL) != SCM_BOOL_T) {
-    char *val = SCM_STRING_CHARS(cascade_value);
+    char *val = scm_to_locale_string(cascade_value);
     ruin_util_hash_insert(tree->style_cache, prop, val);
     return val;
   } else if ((tree->parent == NULL) ||
-	     (scm_call_1(sip_p, scm_prop) == SCM_BOOL_F)) {
+	     ((scm_call_1(sip_p, scm_prop) == SCM_BOOL_F) &&
+	      ((strcmp(ruin_css_lookup(tree->parent, "display"), 
+		       "inline") != 0) ||
+	       (scm_equal_p(tree->parent->element, 
+			    scm_makfrom0str("ruin-inline-element")) == 
+		SCM_BOOL_T)))) { 
+
+    /* The above expression says, "Don't try to inherit a value if the parent
+       node is NULL or if the value isn't inheritable -- UNLESS the parent
+       element is an inline element with no content (i.e., it's a <span> tag
+       or something)." */
+
     char *vc = NULL;
     long s = ruin_util_current_time_millis();
     cascade_value = scm_call_1(sgdv_p, scm_prop);
     full_value_select_time += ruin_util_current_time_millis() - s;
     full_value_select_num++;
-    vc = SCM_STRING_CHARS(cascade_value);
+    vc = scm_to_locale_string(cascade_value);
     ruin_util_hash_insert(tree->style_cache, prop, vc);
     return vc;
   } else {
@@ -235,6 +268,8 @@ char *ruin_css_lookup(ruin_element_t *tree, char *prop) {
 }
 
 void ruin_css_clear_style_cache(ruin_element_t *t) {
+  if (t == NULL)
+    return;
   ruin_util_hash_clear(t->style_cache);
   if (scsc_p == SCM_EOL) {
     scsc_p = scm_c_eval_string("scss:clear-style-cache!");
@@ -242,4 +277,6 @@ void ruin_css_clear_style_cache(ruin_element_t *t) {
   if (scm_string_p(t->element) == SCM_BOOL_T)
     scm_call_2(scsc_p, t->cascade, t->element);
   else scm_call_3(scsc_p, t->cascade, t->element, t->doc);
+  ruin_css_clear_style_cache(t->first_child);
+  ruin_css_clear_style_cache(t->next_sibling);
 }
