@@ -1,5 +1,5 @@
 /* dialect.c: Common routines related to tree generation
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "css.h"
 #include "dialect.h"
 #include "layout.h"
+#include "scheme.h"
 #include "util.h"
 
 SCM ruin_dialect_parse_in_document_style(SCM doc, SCM style) {
@@ -164,16 +165,16 @@ char *ruin_dialect_get_node_name(SCM node) {
     node_name_scm = 
       scm_call_2(scm_c_eval_string("sdom:get-dom-property"), node,
 		 scm_makfrom0str("sdom:tag-name"));
-    node_name = SCM_STRING_CHARS(node_name_scm);
+    node_name = scm_to_locale_string(node_name_scm);
   }
-  else node_name = SCM_STRING_CHARS(local_name_scm);
+  else node_name = scm_to_locale_string(local_name_scm);
   return node_name;  
 }
 
 ruin_element_t *ruin_dialect_generate_text_node(SCM node, 
 						ruin_element_t *p,
 						ruin_element_t *s) {
-  char *content = SCM_STRING_CHARS
+  char *content = scm_to_locale_string
     (scm_call_2(scm_c_eval_string("sdom:get-dom-property"), node, 
 		scm_makfrom0str("sdom:node-value")));
   
