@@ -22,6 +22,7 @@
 #include "modify.h"
 #include "../layout.h"
 #include "../render.h"
+#include "../scheme.h"
 #include "../window.h"
 
 SCM ruin_scm_default_handler_attr_modified(SCM event) {
@@ -29,7 +30,7 @@ SCM ruin_scm_default_handler_attr_modified(SCM event) {
 			  event, scm_makfrom0str("sdom:target"));
   ruin_window_t *win = ruin_window_lookup_scm(target)->parent_window;
   char *ptr_str =
-    SCM_STRING_CHARS(scm_hashq_ref(win->scm_hash, target, SCM_EOL));
+    scm_to_locale_string(scm_hashq_ref(win->scm_hash, target, SCM_EOL));
   ruin_element_t *tree = (ruin_element_t *) ruin_util_string_to_ptr(ptr_str);
   ruin_util_list *l = ruin_util_list_new();
 
