@@ -130,8 +130,8 @@ ruin_element_t *ruin_element_new() {
 
   t->style_cache = ruin_util_hash_new();
 
-  t->inherent_attribute_style = SCM_EOL;
-  t->additional_attribute_style = SCM_EOL;
+  t->inherent_attribute_style = scm_list_n(SCM_UNDEFINED);
+  t->additional_attribute_style = scm_list_n(SCM_UNDEFINED);
   scm_gc_protect_object(t->inherent_attribute_style);
   scm_gc_protect_object(t->additional_attribute_style);
   t->current_pseudo_elements = ruin_util_list_new();
@@ -222,22 +222,25 @@ ruin_layout_size_t ruin_layout_size_text(ruin_element_t *t,
   int i;
   int current_line_length = 0;
   int num_lines = 1;
-  int max_line_length = -1;
+  int max_line_length = 0;
 
   ruin_layout_size_t return_val;
 
   _ruin_layout_parse_sizes(t);
+  ruin_layout_normalize_lengths(t, inheritance, ruin_layout_block_mask);
 
   t->top = top;
   t->left = left;
-  t->first_line_start = left + first_line;
+  t->first_line_start = left + first_line + t->margin_left.used;
+  max_line_length = t->margin_left.used;
 
-  return_val.first_line = 0;
+  return_val.first_line = t->margin_left.used;
   return_val.height = 0;
   return_val.width = 0;
   return_val.last_line = 0;
 
-  current_line_length = first_line + t->parent->text_indent.used;
+  current_line_length = first_line + t->parent->text_indent.used + 
+    t->margin_left.used;
 
   if (t->content != NULL) {
     char **words = NULL;
@@ -250,26 +253,37 @@ ruin_layout_size_t ruin_layout_size_text(ruin_element_t *t,
 	  ((first_line == 0) || last_ended_with_space))
 	word_lens[0]--;
       for (i = 0; i < num_words; i++) {
-	/* fprintf(stderr, "num_words %d i %d\n", num_words, i); */
-	current_line_length += word_lens[i] * (1 + t->letter_spacing.used);
-	if (current_line_length > max_line_length)
-	  max_line_length = current_line_length;
-	
-	/* Did the current word overflow the line box? */
-
-	while(current_line_length > w) {
-	  int remainder = w - current_line_length;
-	  current_line_length -= remainder;	  
-	}
-
-	if (i < (num_words - 1)) {
-	  current_line_length += t->word_spacing.used;
-	  if (current_line_length > w) {
-
-	    /* Did the word-spacing overflow the line box?  If so, ditch it, 
-	       because we don't observe whitespace across lines. */
-	
+	int addition = word_lens[i] * (1 + t->letter_spacing.used);
+	if (w > 0 && (current_line_length + addition > w)) {
+	  if (addition > w) {
+	    addition -= w - current_line_length;
+	    num_lines++;
+	    while(addition > 0) {
+	      addition -= w;
+	      num_lines++;
+	    }
+	    current_line_length = w + addition;
+	  } else {
 	    current_line_length = 0;
+	    num_lines++;
+	  }
+	} else {
+	  current_line_length += addition;
+	  if (current_line_length > max_line_length)
+	    max_line_length = current_line_length;	
+
+	  /* Did the current word overflow the line box? */	
+	  
+	  if (i < (num_words - 1)) {
+	    current_line_length += t->word_spacing.used;
+	    if (w > 0 && (current_line_length > w)) {
+
+	      /* Did the word-spacing overflow the line box?  If so, ditch it, 
+		 because we don't observe whitespace across lines. */
+	      
+	      current_line_length = 0;
+	      num_lines++;
+	    }
 	  }
 	}
       }
@@ -290,7 +304,6 @@ ruin_layout_size_t ruin_layout_size_text(ruin_element_t *t,
     if (current_line_length > max_line_length) 
       max_line_length = current_line_length;
     
-    return_val.first_line = 0;
     return_val.height = ((h >= 0) && (num_lines > h)) ? h : num_lines;
     return_val.width = max_line_length;
   }
@@ -345,7 +358,7 @@ ruin_layout_size_t ruin_layout_size_text(ruin_element_t *t,
 			  last_ended_with_space);
   }
 
-  return_val.last_line = current_line_length;
+  return_val.last_line = current_line_length + t->margin_right.used;
   t->last_line_length = current_line_length;
   t->width.used = return_val.width + 1;
   t->height.used = return_val.height;
@@ -604,15 +617,20 @@ ruin_layout_size_t ruin_layout_size_inline(ruin_element_t *t,
   
   if ((t->prev_sibling != NULL) && 
       (strcmp(ruin_css_lookup(t->prev_sibling, "display"), "inline") == 0)) {
-    first_line_length -= t->prev_sibling->last_line_length;
+    first_line_length -= t->prev_sibling->last_line_length + 
+      t->prev_sibling->border_right_width.used;
   } else if ((t->parent->first_child == t) &&
 	     (strcmp(ruin_css_lookup(t->parent, "display"), "inline") == 0)) {
-    first_line_length -= t->parent->last_line_length;
+    first_line_length -= t->parent->last_line_length +
+      t->parent->border_right_width.used;
   }
+
+  first_line_length -= t->border_left_width.used;
   
   ruin_util_list_push_front(inheritance, ruin_util_ptr_to_string(t));
   return_val = ruin_layout_size_text
-    (t, inheritance, top, left, 0, t->parent->width.used, -1, FALSE);
+    (t, inheritance, top, left, t->border_left_width.used, 
+     t->parent->width.used, -1, FALSE);
   (void) ruin_util_list_pop_front(inheritance);
 
   return return_val;
@@ -1356,7 +1374,7 @@ void ruin_layout_add_style(SCM *list, char *prop, char *value) {
   if (scm_eq_p(*list, SCM_EOL) != SCM_BOOL_T)
     scm_append_x(scm_list_2(SCM_CDR(*list), scm_list_1(addition)));
   else {
-    *list = scm_list_2(scm_list_1(scm_makfrom0str("*")), addition);
+    *list = scm_list_1(scm_list_2(scm_list_1(scm_makfrom0str("*")), addition));
     scm_gc_protect_object(*list);
   }
 }
