@@ -21,7 +21,7 @@
 
 #include <libguile.h>
 
-#include "load.h"
+#include "libruin.h"
 #include "util.h"
 #include "window.h"
 
