@@ -20,6 +20,7 @@
 #include <curses.h>
 #include <libguile.h>
 #include <limits.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -29,6 +30,7 @@
 #include "layout.h"
 #include "load.h"
 #include "render.h"
+#include "scheme.h"
 #include "util.h"
 #include "window.h"
 #include "xhtml.h"
@@ -37,6 +39,7 @@
 #define GUILE_LOAD_PATH "%load-path"
 
 static int ruin_initialized = FALSE;
+static struct sigaction ruin_sigaction_old_action;
 
 ruin_windows_t *_ruin_windows = NULL;
 
@@ -50,24 +53,65 @@ static void _install_default_handler(SCM node, char *event, char *handler) {
 			 scm_c_eval_string(handler), SCM_BOOL_T));
 }
 
+long ruin_load_layout_and_render(ruin_window_t *w) {
+  long start_time = ruin_util_current_time_millis();
+
+  { ruin_util_list *inheritance = ruin_util_list_new();
+    ruin_layout_normalize_lengths(w->top, inheritance, 0);
+    ruin_util_list_push_front(inheritance, ruin_util_ptr_to_string(w->top));
+    ruin_layout_size_tree
+      (w->top->first_child, inheritance, w->top->top, w->top->left);
+    ruin_util_list_free(inheritance);
+  }
+
+  /* We don't set the pseudo-element handler until render time, because
+     otherwise we wind up with a recursive overflow... */
+
+  scm_call_1(scm_c_eval_string("scss:set-pseudo-element-handler!"),
+	     scm_c_eval_string("ruin:scss-pseudo-element-handler"));
+
+  { SCM root = w->top->first_child->element;
+    _install_default_handler
+      (root, "sdom:event-keyup", "ruin:default-tab-handler");
+
+    _install_default_handler(root, "sdom:event-dom-attr-modified", 
+			     "ruin:default-attr-modified-handler");
+    _install_default_handler(root, "sdom:event-dom-node-inserted",
+			     "ruin:default-node-insertion-handler");
+    _install_default_handler(root, "sdom:event-dom-node-removed",
+			     "ruin:default-node-removal-handler");
+  }
+
+  wclear(w->window);
+
+  { ruin_util_list *inh = ruin_util_list_new();
+    ruin_render_render_tree(w->top->first_child, inh);
+    ruin_util_list_free(inh);
+  }
+
+  wrefresh(w->window);
+  return ruin_util_current_time_millis() - start_time;
+}
+
 int ruin_draw(ruin_window_t *w, SCM doc, enum ruin_xml_dialect lang) {
   ruin_element_t *top_level;
   ruin_element_t *tree = NULL;
   int top, left, bottom, right;
-  long start_time;
   
   SCM element;
 
   SCM cascade = scm_call_0(scm_c_eval_string("scss:create-cascade"));
   scm_gc_protect_object(cascade);
     
-  getbegyx(w->window, top, left);
-  getmaxyx(w->window, bottom, right);
-
   /* Generate default stylesheet.  If the document has an attached stylesheet
      or adds style on a per-element basis, we can apply that on top of the 
      default. */
 
+  getbegyx(w->window, top, left);
+  getmaxyx(w->window, bottom, right);
+
+  ruin_util_log(w, "window dimensions are %d, %d", right, bottom);
+
   top_level = ruin_element_new();
   top_level->dialect = lang;
   top_level->cascade = cascade;
@@ -79,6 +123,7 @@ int ruin_draw(ruin_window_t *w, SCM doc, enum ruin_xml_dialect lang) {
 
   top_level->height.computed = bottom - top;
   top_level->width.computed = right - left;
+
   top_level->max_width = top_level->width;
   top_level->max_height = top_level->height;
   top_level->padding_left.computed = 0;
@@ -107,58 +152,16 @@ int ruin_draw(ruin_window_t *w, SCM doc, enum ruin_xml_dialect lang) {
     break;
   }
 
-  top_level->first_child = tree;
-  top_level->first_child->element = element;
-
-  start_time = ruin_util_current_time_millis();
-
-  { ruin_util_list *inheritance = ruin_util_list_new();
-    ruin_layout_normalize_lengths(top_level, inheritance, 0);
-    ruin_util_list_push_front(inheritance, ruin_util_ptr_to_string(top_level));
-    ruin_layout_size_tree
-      (tree, inheritance, top_level->top, top_level->left);
-    ruin_util_list_free(inheritance);
-  }
-
-  /* We don't set the pseudo-element handler until render time, because
-     otherwise we wind up with a recursive overflow... */
-
-  scm_call_1(scm_c_eval_string("scss:set-pseudo-element-handler!"),
-	     scm_c_eval_string("ruin:scss-pseudo-element-handler"));
-
-  { SCM root = top_level->first_child->element;
-    _install_default_handler
-      (root, "sdom:event-keyup", "ruin:default-tab-handler");
-    _install_default_handler
-      (root, "sdom:event-dom-focus-out", "ruin:default-focus-toggle-handler");
-    _install_default_handler
-      (root, "sdom:event-dom-focus-in", "ruin:default-focus-toggle-handler");
-    _install_default_handler(root, "sdom:event-dom-attr-modified", 
-			     "ruin:default-attr-modified-handler");
-    _install_default_handler(root, "sdom:event-dom-node-inserted",
-			     "ruin:default-node-insertion-handler");
-    _install_default_handler(root, "sdom:event-dom-node-removed",
-			     "ruin:default-node-removal-handler");
-  }
-
-  w->focused = (ruin_element_t *)
-    ruin_util_string_to_ptr(ruin_util_list_peek_after(w->tab_order, 0));
-  if (w->focused == NULL)
-    w->focused = top_level->first_child;
-  w->top = top_level->first_child;
-
   scm_call_2(scm_c_eval_string("sdom:dispatch-event"), w->top->element, 
 	     scm_str2symbol("sdom:event-load"));
   
-  { ruin_util_list *inh = ruin_util_list_new();
-    ruin_render_render_tree(top_level->first_child, inh);
-    ruin_util_list_free(inh);
-  }
+  top_level->first_child = tree;
+  top_level->first_child->element = element;
 
-  ruin_util_log(w, "total time %ldms", 
-		ruin_util_current_time_millis() - start_time);
+  w->top = top_level;
+
+  ruin_util_log(w, "total time %ldms", ruin_load_layout_and_render(w));
 
-  refresh();
   return TRUE;
 }
 
@@ -202,11 +205,18 @@ void _set_ruin_temp_load_path(SCM oldval, char *varname) {
 int ruin_init() {
   SCM old_path = scm_list_copy(scm_c_eval_string(GUILE_LOAD_PATH));
   extern pthread_mutex_t _ruin_util_id_lock;
-                
+
+  struct sigaction new_action;
+  new_action.sa_sigaction = ruin_window_signal_handler_SIGWINCH;
+  new_action.sa_flags = SA_SIGINFO;
+  sigemptyset(&new_action.sa_mask);
+
   if (ruin_initialized)
     return FALSE;
 
   pthread_mutex_init(&_ruin_util_id_lock, NULL);
+  (void) sigaction(SIGWINCH, NULL, &ruin_sigaction_old_action);
+  (void) sigaction(SIGWINCH, &new_action, NULL);
 
   scm_c_use_module("ice-9 regex");
 
@@ -225,6 +235,7 @@ int ruin_init() {
 
   /* Also initialize our own Scheme API. */
 
+  ruin_scheme_init();
   _ruin_scm_init_api();
   _ruin_scm_init_handlers();
   
@@ -241,6 +252,8 @@ int ruin_init() {
 			       "(sdom:get-dom-property n \"sdom:node-type\") "
 			       "sdom:node-type-element) (equal? s "
 			       "(sdom:get-attribute n \"id\"))))"));
+  scm_call_1(scm_c_eval_string("scss:set-pseudo-class-handler!"),
+	     scm_c_eval_string("ruin:scss-pseudo-class-handler"));
 
   /* Set up a default color pair... */
   init_pair(1, COLOR_WHITE, COLOR_BLACK);
@@ -255,5 +268,6 @@ void ruin_shutdown() {
     return;
   pthread_mutex_destroy(&_ruin_util_id_lock);
   ruin_windows_free(_ruin_windows);
+  (void) sigaction(SIGWINCH, &ruin_sigaction_old_action, NULL);
   return;
 }
