@@ -19,9 +19,8 @@
 #ifndef RUIN_LOAD_H
 #define RUIN_LOAD_H
 
-enum ruin_xml_dialect {
-  RUIN_XML_DIALECT_XUL,
-  RUIN_XML_DIALECT_XHTML
-};
+#include "libruin.h"
 
-#endif
+long ruin_load_layout_and_render(ruin_window_t *);
+
+#endif 
