@@ -250,39 +250,43 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
   if ((max_line_length <= 0) || (max_lines <= 0))
     return;
 
-  if (tree->content != NULL) {
+  if (tree->content != NULL &&
+      (num_words = ruin_render_get_words
+       (tree->content, &words, &word_lens)) > 0) {
     char *wstype = ruin_css_lookup(tree, "white-space");
-    num_words = ruin_render_get_words(tree->content, &words, &word_lens);
     
     current_line_length = 0;
     current_line_words = 0;
-    num_lines = 0;
+    num_lines = 1;
 
     lines = malloc(sizeof(char *) * tree->height.used);
     
     if (strcmp(wstype, "normal") == 0) {
       char *buf = calloc(tree->width.used + 1, sizeof(char));
       int buf_len = tree->width.used;
-      int num_words = ruin_render_get_words
-	(tree->content, &words, &word_lens);
+      int num_words = ruin_render_get_words(tree->content, &words, &word_lens);
 
       lines[current_line] = buf;
+
       if ((isspace(words[0][0])) && (current_line == 0)) {
 	word_lens[0]--;
 	words[0]++;
       }
+
       if ((tree->first_child == NULL) && (tree->next_sibling == NULL) &&
 	  (isspace(words[num_words - 1][word_lens[num_words - 1] - 1]))) {
 	words[num_words - 1][word_lens[num_words - 1] - 1] = 0;
 	word_lens[num_words - 1]--;
       }
+
       for (i = 0; i < num_words; i++) {
 	int wlen = word_lens[i];
-	int total_wlen = wlen * ls - 1;
+	int total_wlen = wlen * (ls + 1) - (ls > 0 ? 1 : 0);
 	int addition = 0;
 	int ws_p = i == num_words - 1 ? 0 : ws;
 	
 	if (current_line_length + total_wlen > tree->width.used) {
+	  num_lines++;
 
 	  /* The current word won't fit on the current line... Will it fit on
 	     any line? */
@@ -293,10 +297,20 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 
 	    int remainder = total_wlen;
 	    while (remainder > 0) {
-	      /* addition = _ruin_render_add_word(buf, buf_len, words[i] + */
+	      addition = _ruin_render_add_word
+		(buf, buf_len, words[i] + total_wlen - remainder, buf_len, ls, 
+		 ws_p);
 	      buf += addition;
-	      buf_len -= addition;
+	      buf_len -= addition;	
 	      remainder -= addition;
+	      if (buf_len == 0) {
+		if (current_line < tree->height.used - 1) {
+		  buf = calloc(tree->width.used, sizeof(char));
+		  lines[++current_line] = buf;
+		  buf_len = tree->width.used;
+		  current_line_length = 0;
+		} else break;
+	      }
 	    }
 
 	  } else {
@@ -322,6 +336,20 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 	}
       }
     } else if (strcmp(wstype, "pre") == 0) {
+      char *br = index(tree->content, '\n');
+      char *oldbr = tree->content;
+      if (br == NULL) {
+	lines[num_lines - 1] = strdup(tree->content);
+      } else {
+	do {
+	  int len = br - oldbr;
+	  char *buf = malloc(sizeof(char) * (len + 1));
+	  strncpy(buf, br, len);
+	  lines[num_lines - 1] = buf;
+	  num_lines++;
+	  oldbr = br;
+	} while ((br = index(br + sizeof(char), '\n')) != NULL);
+      }
     } else if (strcmp(wstype, "nowrap") == 0) {
     } else if (strcmp(wstype, "pre-wrap") == 0) {
     } else if (strcmp(wstype, "pre-line") == 0) {
