@@ -1944,11 +1944,14 @@
 
 (define sdom:create-document
   (lambda (root-name doctype . namespace-uri)
-    (letrec ((newdoc (list '*TOP* 
-			   (list (string->symbol root-name)
-				 (list '@ 
-				  (list '@ (list 'sdom:parent-node 
-						 (lambda () newdoc))))))))
+    (let* ((head (list '*TOP*))
+	   (newdoc (if (not (null? root-name))
+		       (append head
+			       (list (string->symbol root-name)
+				     (list '@ 
+				      (list '@ (list 'sdom:parent-node 
+						     (lambda () head))))))
+		       head)))
       (if (not (null? doctype))
 	  (begin
 	    (if (not (eqv? (sdom:node-type doctype) 
