@@ -17,6 +17,7 @@
  */
 
 #include <ctype.h>
+#include <libguile.h>
 #include <math.h>
 #include <pthread.h>
 #include <stdarg.h>
@@ -592,7 +593,7 @@ void ruin_util_log(ruin_window_t *w, char *msg, ...) {
   va_list args;
   va_start(args, msg);
   if (w->log != NULL) {
-    fprintf(w->log, "window %d: ", w->internal_id);
+    fprintf(w->log, "window %ld: ", w->internal_id);
     vfprintf(w->log, msg, args);
     fflush(w->log); 
   }
@@ -642,3 +643,7 @@ long ruin_util_current_time_millis() {
   (void) gettimeofday(&tv, NULL);
   return (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
 }
+
+SCM ruin_util_string2scm(char *str) {
+  return scm_mem2string(str, strlen(str));
+}
