@@ -16,15 +16,69 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 
+#ifdef LINUX
+#include <pty.h>
+#endif /* LINUX */
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/ioctl.h>
 
 #include "layout.h"
+#include "load.h"
+#include "scheme.h"
 #include "util.h"
 #include "window.h"
 
 extern ruin_windows_t *_ruin_windows;
 
+extern long ruin_load_layout_and_render(ruin_window_t *);
+
+static pthread_mutex_t ruin_window_signal_handler_SIGWINCH_entry_lock;
+
+void ruin_window_signal_handler_SIGWINCH(int n, siginfo_t *s, void *u) {
+  static int busy = 0;
+  int num_keys = 0, i = 0;
+  int oldx = 0, oldy = 0, newx = 0, newy = 0;
+  int iocx = 0, iocy = 0;
+  char **keys = NULL;
+  struct winsize ws;
+
+  pthread_mutex_lock(&ruin_window_signal_handler_SIGWINCH_entry_lock);
+  if (busy) {
+    pthread_mutex_unlock(&ruin_window_signal_handler_SIGWINCH_entry_lock);
+    return;
+  }
+  pthread_mutex_unlock(&ruin_window_signal_handler_SIGWINCH_entry_lock);
+  busy = TRUE;
+
+  keys = ruin_util_hash_get_keys(_ruin_windows->windows, &num_keys);
+
+  (void) ioctl(0, TIOCGWINSZ, &ws);
+  iocx = ws.ws_col;
+  iocy = ws.ws_row;
+
+  for (i = 0; i < num_keys; i++) {
+    ruin_window_t *w = (ruin_window_t *) ruin_util_string_to_ptr 
+      (ruin_util_hash_retrieve(_ruin_windows->windows, keys[i]));
+
+    oldx = w->top->width.used;
+    oldy = w->top->height.used;
+    
+    newy = iocy;
+    newx = iocx;
+
+    if (oldx != newx || oldy != newy) {
+      w->top->max_width.computed = w->top->width.computed = newx;
+      w->top->max_height.computed = w->top->height.computed = newy;
+
+      resizeterm(newy, newx); 
+      ruin_load_layout_and_render(w);
+    }
+  }
+  busy = FALSE;
+}
+
 ruin_windows_t *ruin_windows_new() {
   char *xul = NULL;
   char *xhtml = NULL;
@@ -55,6 +109,8 @@ ruin_windows_t *ruin_windows_new() {
 
   free(xul);
   free(xhtml);
+
+  pthread_mutex_init(&ruin_window_signal_handler_SIGWINCH_entry_lock, NULL);
   return t;
 }
 
@@ -131,7 +187,7 @@ ruin_element_t *ruin_window_lookup_scm(SCM elt) {
 
   if (containing_win != NULL)
     return (ruin_element_t *) ruin_util_string_to_ptr
-      (SCM_STRING_CHARS
+      (scm_to_locale_string
        (scm_hashq_ref(containing_win->scm_hash, elt, SCM_EOL)));
   return NULL;  
 }
