@@ -21,6 +21,7 @@
 
 #include <curses.h>
 #include <libguile.h>
+#include <signal.h>
 #include <stdio.h>
 
 #include "layout.h"
@@ -32,7 +33,7 @@
 #define RUIN_WINDOW_DEFAULT_FONT_WIDTH   8
 #define RUIN_WINDOW_DEFAULT_DPI         96
 
-typedef struct _ruin_window_t {
+struct _ruin_window_t {
   WINDOW *window;
   FILE *log;
 
@@ -52,7 +53,7 @@ typedef struct _ruin_window_t {
   struct _ruin_element_t *top;
 
   short render_requested;
-} ruin_window_t;
+};
 
 typedef struct _ruin_windows_t {
   struct _ruin_util_hash *windows;
@@ -71,4 +72,6 @@ struct _ruin_element_t *ruin_window_lookup_scm(SCM elt);
 /* Because I'm a moron and can't figure out how else to prototype this... */
 void ruin_util_log(ruin_window_t *, char *, ...);
 
+void ruin_window_signal_handler_SIGWINCH(int, siginfo_t *, void *);
+
 #endif
