@@ -1,5 +1,5 @@
 /* xul.c: XUL-parsing support for libRUIN
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2006 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "css.h"
 #include "dialect.h"
 #include "layout.h"
+#include "scheme.h"
 #include "util.h"
 #include "window.h"
 #include "xul.h"
@@ -54,9 +55,9 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
     plocal_name_scm =  
       scm_call_2(scm_c_eval_string("sdom:get-dom-property"), parent, 
 		 scm_makfrom0str("sdom:tag-name"));
-    parent_name = SCM_STRING_CHARS(plocal_name_scm);
+    parent_name = scm_to_locale_string(plocal_name_scm);
   }
-  else parent_name = SCM_STRING_CHARS(plocal_name_scm);
+  else parent_name = scm_to_locale_string(plocal_name_scm);
 
   /* First we need to check the namespaces... */
 
@@ -66,18 +67,18 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
     local_name_scm = scm_call_2
       (scm_c_eval_string("sdom:get-dom-property"), node, 
        scm_makfrom0str("sdom:name"));
-    attr_name = SCM_STRING_CHARS(local_name_scm);
+    attr_name = scm_to_locale_string(local_name_scm);
   }
   else {
-    attr_name = SCM_STRING_CHARS(local_name_scm);
+    attr_name = scm_to_locale_string(local_name_scm);
     ns_name_scm = scm_call_2(scm_c_eval_string("sdom:get-dom-property"),
 			     node, scm_makfrom0str("sdom:namespace-uri"));
-    attr_ns = SCM_STRING_CHARS(ns_name_scm);
+    attr_ns = scm_to_locale_string(ns_name_scm);
   }
   
   attr_str_scm = scm_call_2(scm_c_eval_string("sdom:get-dom-property"), node,
 			    scm_makfrom0str("sdom:value"));
-  attr_str = SCM_STRING_CHARS(attr_str_scm);
+  attr_str = scm_to_locale_string(attr_str_scm);
 
   if (attr_ns != NULL) {
     if (strcmp((char *) attr_ns, "http://www.w3.org/TR/CSS21") == 0)
